!***********************************************************************
!*****************************  KEK, High Energy Accelerator Research  *
!*****************************  Organization                           *
!****** u c s h i e l d ******                                         *
!*****************************    EGS5.0 USER CODE - 26 Sep 2014/1600 *
!***********************************************************************
!***********************************************************************
!                                                                      *
!  PROGRAMMERS:  H. Hirayama                                           *
!                Applied Research Laboratory                           *
!                KEK, High Energy Accelerator Research Organization    *
!                1-1, Oho, Tsukuba, Ibaraki, 305-0801                  *
!                Japan                                                 *
!                                                                      *
!                E-mail:      hideo.hirayama@kek.jp                    *
!                Telephone:   +81-29-864-5451                          *
!                Fax:         +81-29-864-4051                          *
!                                                                      *
!                Y. Namito                                             *
!                Radiation Science Center                              *
!                Applied Research Laboratory                           *
!                KEK, High Energy Accelerator Research Organization    *
!                1-1, Oho, Tsukuba, Ibaraki, 305-0801                  *
!                Japan                                                 *
!                                                                      *
!                E-mail:      yoshihito.namito@kek.jp                  *
!                Telephone:   +81-29-864-5489                          *
!                Fax:         +81-29-864-1993                          *
!                                                                      *
!***********************************************************************
!***********************************************************************
!
! This user code corresponds to ucshield4.mor for egs4.
! egs5 user's code to output particle trajectory in Al, Fe, and Pb.
! The following shows the geometry
!***********************************************************************
!                                                                      *
!             -------------------------------------------------        *
!             1-Dimensional Plane Z Geometry (ucsampl5 example)        *
!             -------------------------------------------------        *
!                                                                      *
!                Y (X into page)                                       *
!                ^                                                     *
!                |                                                     *
!                |     |      |     |                                  *
!                |     | Al   |     |                                  *
!                | Air | Fe   | Air |                                  *
!                |     | Pb   |     |                                  *
!     particle   |     | PMMA |     |                                  *
!      ==========>+----+------+-----+-----------> Z                    *
!                0    5.0    zthick                                    *
!                                                                      *
!***********************************************************************
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12
!-----------------------------------------------------------------------
!------------------------------- main code -----------------------------
!-----------------------------------------------------------------------

!-----------------------------------------------------------------------
!     Step 1. Initialization
!-----------------------------------------------------------------------

      implicit none

!     ------------
!     EGS5 COMMONs
!     ------------
      include 'include/egs5_h.f'                ! Main EGS "header" file

      include 'include/egs5_bounds.f'
      include 'include/egs5_edge.f'
      include 'include/egs5_media.f'
      include 'include/egs5_misc.f'
      include 'include/egs5_thresh.f'
      include 'include/egs5_useful.f'
      include 'include/egs5_usersc.f'
      include 'include/egs5_userxt.f'
      include 'include/randomm.f'

!     ----------------------
!     Auxiliary-code COMMONs
!     ----------------------
      include 'auxcommons/aux_h.f'   ! Auxiliary-code "header" file

      include 'auxcommons/georz.f'
      include 'auxcommons/lines.f'
      include 'auxcommons/nfac.f'
      include 'auxcommons/pladta.f'

      common/totals/esum(6)
      real*8 esum

      real*8 ei,ekin,etot,totke,xi,yi,zi,   ! Arguments
     *       ui,vi,wi,wti

      real*8 ct,ct0,phai,rphai,zair,zthick   ! Local variables

      integer i,idinc,iii,iplsrc,iqi,iri,isorsc,ismat,ixray,
     *        j,ncases,mode
      integer nplanx,nplany,nplanz,nregcv

      character*24 medarr(5)

!      open(7,file='egs5job.out',status='unknown')

!     ====================
      call counters_out(0)
!     ====================

      mode=1
!          0: include pegs
!          1: use material data created by mode=0

      npreci=3    ! 0:precision of x-, y-, z-position in 4 digits   PICT
                  ! 1:precision of x-, y-, z-position in 8 digits   PICT
                  ! 0,1: for EGS4WIN32
                  ! 2,3: for CGview 
                  ! 2: fixed format
                  ! 3: free format
                   

!-----------------------------------------------------------------------
! Step 2: pegs5-call
!-----------------------------------------------------------------------
!     ==============
      call block_set                 ! Initialize some general variables
!     ==============

!     ---------------------------------
!     define media before calling PEGS5
!     ---------------------------------

      nmed=5
      medarr(1)='AL                      '
      medarr(2)='FE                      '
      medarr(3)='PB                      '
      medarr(4)='AIR AT NTP              '
      medarr(5)='PMMA                    '

      do j=1,nmed
        do i=1,24
          media(i,j)=medarr(j)(i:i)
        end do
      end do  

      chard(1) = 1.0
      chard(2) = 1.0
      chard(3) = 1.0
      chard(4) = 5.0
      chard(5) = 1.0

!     ------------------------------
!     Run PEGS5 before calling HATCH
!     ------------------------------
      if(mode.eq.0) then
        write(6,'(A)') ' PEGS5-call comes next'

!       =============
        call pegs5
!       =============
      end if

!-----------------------------------------------------------------------
! Step 3: Pre-hatch-call-initialization
!-----------------------------------------------------------------------
1000  write(6,'(A/A)') ' Key in Material number ',
     *       ' (1:Al, 2:Fe, 3:Pb, 4:Air, 5:PMMA, 0:end)'
      read(5,*) ismat
 
      if(ismat.eq.0) go to 9999

      nreg=6

      med(1)=0     ! Region 1 is vacuum
      med(2)=4     ! Region 2 is air
      med(3)=ismat ! 1:Al, 2:Fe, 3:Pb, 4:Air, 5:PMMA
      med(4)=4     ! Region 4 is air
      med(5)=0     ! Region 5 is vacuum
      med(6)=0     ! Region 6 is vacuum

!     ----------------------------------
!     Set of option flag for region 2-3
!     1: on, 0: off
!     ----------------------------------
      
      write(6,'(A/A)') ' Do you want to produce fluorescence X-ray ?',
     *       '  (0:no, 1:yes)'
      read(5,*) ixray


      do i=2,nreg
        iphter(i) = 0       ! Switches for PE-angle sampling
        iedgfl(i) = ixray   ! K & L-edge fluorescence
        iauger(i) = 0       ! K & L-Auger
        iraylr(i) = 0       ! Rayleigh scattering
        lpolar(i) = 0       ! Linearly-polarized photon scattering
        incohr(i) = 0       ! S/Z rejection
        iprofr(i) = 0       ! Doppler broadening
        impacr(i) = 0       ! Electron impact ionization
      end do

!-----------------------------------------------------------------------
! Step 4:   hatch-call  Ordinary at Step 5
!-----------------------------------------------------------------------
      emaxe = 0.d0

!     --------------------------------------------------------
!     Random number seeds.  Must be defined before call hatch.
!     ins (1- 2^31)
!     --------------------------------------------------------
!!      inseed=1
!!      luxlev=1

!     =============
!!    call rluxinit   ! Initialize the Ranlux random-number generator
!     =============

!     ------------------------------
!     Open files (before HATCH call)
!     ------------------------------
      if(mode.eq.0) then
        open(UNIT=KMPI,FILE='pgs5job.pegs5dat',STATUS='old')
      else
        open(UNIT=KMPI,FILE='shield.dat',STATUS='old')
      end if

      open(UNIT=KMPO,FILE='egs5job.dummy',STATUS='unknown')

      write(6,'(/A/)') ' HATCH-call comes next'

!     ==========
      call hatch
!     ==========

!     ------------------------------
!     Close files (after HATCH call)
!     ------------------------------
      close(UNIT=KMPI)
      close(UNIT=KMPO)

! ----------------------------------------------------------
! Print various data associated with each media (not region)
! ----------------------------------------------------------
      write(6,'(/A)') ' Quantities associated with each MEDIA:'
      do j=1,nmed
        write(6,'(/1X,24A1)') (media(i,j),i=1,24)
        write(6,'(5X,A,G15.7,A,G15.7,A)')
     *     ' rho=',rhom(j),' g/cu.cm     rlc=',rlcm(j),' cm'
        write(6,'(5X,A,G15.7,A,G15.7,A)') 
     *     ' ae=',ae(j),' MeV    ue=',ue(j),' MeV'
        write(6,'(5X,A,G15.7,A,G15.7,A/)') 
     *     ' ap=',ap(j),' MeV    up=',up(j),' MeV'
      end do

!-----------------------------------------------------------------------
! Step 5:  Determination-of-incident-particle-parameters
!          Ordinary st Step 4
!-----------------------------------------------------------------------
1100  write(6,'(A/A)') ' Key in Particle type',
     *' (-1:electron, 0:photon, 1:positron, 9:new material)'
      read(5,*) iqi  

      if(iqi.eq.9) go to 1000

1200  write(6,'(A/A)') ' Key in Particle Kinetic Energy in MeV',
     *'     (0.0 means new particle)'

      read(5,*) ekin
      if(ekin.eq.0.0) go to 1100

      ei=ekin
      if(iqi.ne.0) ei=ekin+RM

!     if(etot.gt.ue(ismat)) then
      if(ei.gt.ue(ismat)) then
        write(6,'(A,G10.3,A,G10.3,A)') 
     *     ' Total energy ', ei, ' must be less or= ', ue(ismat),'MeV'
        go to 1200
      end if
      if(ekin.gt.up(ismat)) then
        write(6,'(A,G10.3,A)') 
     *     ' Kinetic energy must be less or eqal to ', up(ismat),'MeV'
        go to 1200
      end if

!     --------------------------------------------------------
!     Random number seeds.  Must be defined before call hatch.
!     ins (1- 2^31)
!     --------------------------------------------------------
      inseed=1
      luxlev=1

      kount=0  ! must be set to produce same random number
      mkount=0

!     =============
      call rluxinit   ! Initialize the Ranlux random-number generator
!     =============

      xi=0.0
      yi=0.0
      zi=0.0
      ui=0.0
      vi=0.0
      wi=1.0
      iri=2
      wti=1.0
      ncases=50
      idinc=-1

!-----------------------------------------------------------------------
! Step 6:  Initialization-for-howfar
!-----------------------------------------------------------------------
      zair=5.0    ! Air thickness before and after slab in cm

1300  write(6,'(A)') 
     *   ' Key in slab thickness in cm (0.0 means change energy)'
      read(5,*) zthick   ! Slab thickness in cm
      if(zthick.eq.0.0) go to 1200

      iii=1
      
      nplan=8
      nplanx=2
      nplany=2
      nplanz=4
      do i=1,nplan
        do j=1,3
          pnorm(j,i)=0.d0
          pcoord(j,i)=0.d0
        end do
      end do

      pnorm(3,1)=1.0
      pnorm(3,2)=1.0
      pnorm(3,3)=1.0
      pnorm(3,4)=1.0

      pcoord(3,2)=pcoord(3,1)+zair
      pcoord(3,3)=pcoord(3,2)+zthick
      pcoord(3,4)=pcoord(3,3)+zair

      pnorm(2,5)=1.0
      pnorm(2,6)=1.0
      pcoord(2,5)=-5.0
      pcoord(2,6)=5.0
      pnorm(1,7)=1.0
      pnorm(1,8)=1.0
      pcoord(1,7)=-5.0
      pcoord(1,8)=5.0

      open(39,file='egs5job.pic',status='unknown')     ! PICT

!  Parameter to define graphic size. It is better that the   **PICT**
!  width of each axis is nearly same.                        **PICT**
!     --------------------------------------------
!     Write plane information for geometry drawing
!     --------------------------------------------
      if(npreci.eq.1) then 
!     ------------------------
!     Set parameter for PICT32
!     ------------------------
        xmin=-5.d0
        xmax=5.d0
        ymin=-5.d0
        ymax=5.d0
        zmin=-1.d0
        zmax=zthick+2.0*zair+2.0           ! PICT

        call geomout(0,nplan)                            ! PICT
        fnorm=dmax1(xmax-xmin,ymax-ymin,zmax-zmin)       ! PICT
        write(39,'(7E10.3)') xmin,xmax,ymin,ymax,zmin,zmax,fnorm  ! PICT
      else
!     ------------------------
!     Set parameter for CGVIEW
!     ------------------------
        if(npreci.eq.2) write(39,fmt="('GSTA-TIME')")
        if(npreci.eq.3) write(39,fmt="('GSTA-FREE-TIME')")
        write(39,'(A)') 'SLAB'
        write(39,'(3I6)') nplanx,nplany,nplanz
        write(39,'(4F15.4)') (pcoord(1,nplany+nplanz+j),j=1,nplanx)
        write(39,'(4F15.4)') (pcoord(2,nplanz+j),j=1,nplany)
        write(39,'(4F15.4)') (pcoord(3,j),j=1,nplanz)
        write(39,'(A)') 'GEND'
        
        write(39,'(A)') 'MSTA'
        nregcv=(nplanx-1)*(nplany-1)*(nplanz-1)+3
        if(nreg.ge.nregcv) then
          write(39,'(I4)') nreg
          write(39,'(15I4)') (med(i),i=1,nreg)
        else
          do i=nreg+1,nregcv
            med(i)=0
          end do
          write(39,'(I4)') nregcv
          write(39,'(15I4)') (med(i),i=1,nregcv)
        end if
        write(39,'(A)') 'MEND'
      end if

!-----------------------------------------------------------------------
! Step 7:  Initialization-for-ausgab
!-----------------------------------------------------------------------
      isorsc=0    ! 0:monodirectional, 1:point isotropic.

      iplsrc=1    ! 0:beam, 1:plane source


      do i=1,nreg
        esum(i)=0.D0
      end do

      ilines = 0
      nwrite = 10
      nlines = 100
      ncount = 0

!-----------------------------------------------------------------------
! Step 8:  Shower-call
!-----------------------------------------------------------------------

      ct0=5.0/dsqrt(5.d0*5.d0+5.d0*5.d0)

      write(39,'(A,I5)') '0',iii             ! PICT

      do i=1,ncases

        if(isorsc.eq.1) then  ! Isotropic Source
          call randomset(ct)
          wi=1.0-ct*(1.0-ct0)
          call randomset(rphai)
          phai=3.141569*2.0*rphai
          ui=dcos(phai)*dsqrt(1.d0-wi*wi)
          vi=dsin(phai)*dsqrt(1.d0-wi*wi)
        end if

        if(iplsrc.eq.1) then   ! Plane Source
          call randomset(ct)
          xi=(2.0*ct-1.0)*5.0
          call randomset(ct)
          yi=(2.0*ct-1.0)*5.0
        end if

        ncount=ncount+1

!        if (ncount .le. nwrite .and. ilines .le. nlines) then
!          ilines = ilines + 1
!          write(6,900) ei,xi,yi,zi,ui,vi,wi,iqi,iri,idinc
!900       FORMAT(4G15.7/3G15.7,3I5)
!        end if

        call shower(iqi,ei,xi,yi,zi,ui,vi,wi,iri,wti)

      end do

      write(39,'(A)') '9'                              ! PICT

      call plotxyz(99,0,0,0.D0,0.D0,0.D0,0.D0,0,0.D0,0.D0)     ! PICT
 
      close(39)

      go to 1300

!-----------------------------------------------------------------------
! Step 9:  Output-of-results
!-----------------------------------------------------------------------

9999  stop
      end
!-------------------------last line of main code------------------------
!-------------------------------ausgab.f--------------------------------
! Version:   050701-1615
! Reference: SLAC-R-730, KEK-2005-8 (Appendix 2)
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12
! ----------------------------------------------------------------------
! Required subroutine for use with the EGS5 Code System
! ----------------------------------------------------------------------
! A simple AUSGAB to:
!
!   1) Score energy deposition
!   2) Print out trajectory information
!
! ----------------------------------------------------------------------

      subroutine ausgab(iarg)

      implicit none

      include 'include/egs5_h.f'                ! Main EGS "header" file

      include 'include/egs5_epcont.f'    ! COMMONs required by EGS5 code
      include 'include/egs5_stack.f'
      include 'auxcommons/lines.f'

      common/totals/esum(6)
      real*8 esum

      integer iarg                                           ! Arguments

!     ----------------------
!     Add deposition energy
!     ----------------------

      esum(ir(np))=esum(ir(np)) + edep

!     ------------------------------------
!     Output particle information for plot
!     ------------------------------------
      call plotxyz(iarg,np,iq(np),x(np),y(np),z(np),e(np),ir(np),
     *     wt(np),time(np))

!     ----------------------------------------------------------------
!     Print out stack information (for limited number cases and lines)
!     ----------------------------------------------------------------
!      if (ncount .le. nwrite .and. ilines .le. nlines) then
!        ilines = ilines + 1
!        write(7,101) e(np),x(np),y(np),z(np),u(np),v(np),w(np),
!     *               iq(np),ir(np),iarg
! 101    FORMAT(4G15.7/3G15.7,3I5)
!      end if

      return
      end
!--------------------------last line of ausgab.f------------------------
!-------------------------------howfar.f--------------------------------
! Version:   050701-1615
! Reference: SLAC-R-730, KEK-2005-8 (Appendix 2)
!-----------------------------------------------------------------------
!23456789|123456789|123456789|123456789|123456789|123456789|123456789|12
! ----------------------------------------------------------------------
! Required (geometry) subroutine for use with the EGS5 Code System
! ----------------------------------------------------------------------
! This is a 3-dimensional plane geometry. 
! ----------------------------------------------------------------------

      subroutine howfar

      implicit none

      include 'include/egs5_h.f'                ! Main EGS "header" file

      include 'include/egs5_epcont.f'    ! COMMONs required by EGS5 code
      include 'include/egs5_misc.f'
      include 'include/egs5_stack.f'

      include 'auxcommons/aux_h.f'   ! Auxiliary-code "header" file

      include 'auxcommons/pladta.f'

      integer irl,npl1,npl2,nrg1,nrg2    ! Local variables

      irl=ir(np)   ! Set local variable

      if (irl.le.1.or.irl.ge.nreg-1) then
        idisc = 1
        return
      end if

      npl1=irl
      npl2=irl-1
      nrg1=irl+1
      nrg2=irl-1

      call plan2p(npl1,nrg1,1,npl2,nrg2,-1)

      call plan2p(6,6,1,5,6,-1)

      call plan2p(8,6,1,7,6,-1)

      return
      end
!--------------------------last line of howfar.f------------------------
